﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Nikse.SubtitleEdit.Logic;
using Nikse.SubtitleEdit.Logic.SubtitleFormats;

namespace Nikse.SubtitleEdit.Forms
{
    public partial class Export3DSubStationAlpha : Form
    {

        private Subtitle _subtitle;
        private SubtitleFormat _format;
        private string _fileName;
        public Subtitle Subtitle3D { get; private set; }

        public Export3DSubStationAlpha()
        {
            InitializeComponent();
        }

        internal void Initialize(Subtitle subtitle, SubtitleFormat format, string fileName, VideoInfo videoInfo)
        {
            _fileName = fileName;
            _format = format;
            _subtitle = subtitle;
            comboBoxResolution.SelectedIndex = 0;

            buttonOK.Text = Configuration.Settings.Language.General.OK;
            buttonCancel.Text = Configuration.Settings.Language.General.Cancel;

            if (videoInfo != null && videoInfo.Height > 0 && videoInfo.Width > 0)
            {
                comboBoxResolution.Items[8] = videoInfo.Width + "x" + videoInfo.Height;
                comboBoxResolution.SelectedIndex = 8;
            }
        }

        private void GetResolution(ref int width, ref int height)
        {
            if (comboBoxResolution.SelectedIndex == 1)
            {
                width = 1440;
                height = 1080;
            }
            else if (comboBoxResolution.SelectedIndex == 2)
            {
                width = 1280;
                height = 720;
            }
            else if (comboBoxResolution.SelectedIndex == 3)
            {
                width = 960;
                height = 720;
            }
            else if (comboBoxResolution.SelectedIndex == 4)
            {
                width = 848;
                height = 480;
            }
            else if (comboBoxResolution.SelectedIndex == 5)
            {
                width = 720;
                height = 576;
            }
            else if (comboBoxResolution.SelectedIndex == 6)
            {
                width = 720;
                height = 480;
            }
            else if (comboBoxResolution.SelectedIndex == 7)
            {
                width = 640;
                height = 352;
            }
            else if (comboBoxResolution.SelectedIndex == 8)
            {
                string[] arr = comboBoxResolution.Text.Split('x');
                width = int.Parse(arr[0]);
                height = int.Parse(arr[1]);
            }
            else
            {
                width = 1920;
                height = 1080;
            }
        }

        private void buttonCustomResolution_Click(object sender, EventArgs e)
        {
            ChooseResolution cr = new ChooseResolution();
            if (cr.ShowDialog(this) == DialogResult.OK)
            {
                comboBoxResolution.Items[8] = cr.VideoWidth + "x" + cr.VideoHeight;
                comboBoxResolution.SelectedIndex = 8;
            }
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            int width = 0;
            int height = 0;
            GetResolution(ref width, ref height);
            var sub = new Subtitle();
            foreach (Paragraph p in _subtitle.Paragraphs)
            {                 
                var left = new Paragraph(p);
                var right = new Paragraph(p);

                int singleWidth = width / 2;


                Font font;
                try
                {
                    var fontStyle = FontStyle.Regular;
                    //if (parameter.SubtitleFontBold)
                    //    fontStyle = FontStyle.Bold;
                    font = new Font("Arial", 20, fontStyle);
                }
                catch (Exception exception)
                {
                    MessageBox.Show(exception.Message);
                    font = new Font(FontFamily.Families[0].Name, 20);
                }
                var textWidth = TextDraw.MeasureTextWidth(font, p.Text, false);

                int singleLeftMargin = (int)((singleWidth - textWidth) / 2);

                // ASS/SSA position: {\pos(<x>, <y>)}
                int y = height - 50;
                left.Text = "{\\pos(" + singleLeftMargin + "," + y + "}" + p.Text;
                right.Text = "{\\pos(" + (singleLeftMargin + singleWidth) + "," + y + "}" + p.Text;
                sub.Paragraphs.Add(left);
                sub.Paragraphs.Add(right);
            }
            Subtitle3D = sub;
            DialogResult = DialogResult.OK;
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
        }

    }
}
