﻿using System;
using System.Collections.Specialized;
using System.IO;

namespace Nikse.SubtitleEdit.Logic.Networking
{
    public class SwgServer
    {
        private NetworkServer _myServer;
        private UserList _users;
        private string _fileName;
        private long _fileSize;
        private Nikse.SubtitleEdit.Forms.Networking _receptor;

        public enum Operations
        {
            NULL = 0,
            VER = 1,
            USR = 2,
            JOI = 3,
            LST = 4,
            BYE = 5,
            FIL = 6,
            MSG = 7,
            SST = 8,
            SET = 9,
            INI = 10,
            REQ = 11,
            DEL = 12,
            SHA = 13,
            PLA = 14,
            POS = 15,
            TAK = 16,
        }

        public SwgServer(Nikse.SubtitleEdit.Forms.Networking myreceptor, int port, string newfilename)
        {
            _myServer = new NetworkServer(port);
            _myServer.OnReceive += Receive;
            _myServer.OnClientDisconnect += ClientDisconnects;
            _users = new UserList();
            _receptor = myreceptor;

            this._fileName = string.Empty;
            this._fileSize = 0;

            if (!string.IsNullOrEmpty(newfilename) && File.Exists(newfilename))
            {
                FileInfo finfo = new FileInfo(newfilename);
                _fileSize = finfo.Length;
                this._fileName = finfo.Name;
            }
        }

        public static Operations opToEnum(NetworkOperation operation)
        {
            switch (operation.Name)
            {
                case "VER": return Operations.VER;
                case "USR": return Operations.USR;
                case "JOI": return Operations.JOI;
                case "LST": return Operations.LST;
                case "BYE": return Operations.BYE;
                case "FIL": return Operations.FIL;
                case "MSG": return Operations.MSG;
                case "SST": return Operations.SST;
                case "SET": return Operations.SET;
                case "INI": return Operations.INI;
                case "REQ": return Operations.REQ;
                case "DEL": return Operations.DEL;
                case "SHA": return Operations.SHA;
                case "PLA": return Operations.PLA;
                case "POS": return Operations.POS;
                case "TAK": return Operations.TAK;            
            }
            return Operations.NULL;
        }

        private void Receive(int id, NetworkOperation operation)
        {
            switch (opToEnum(operation))
            {
                case Operations.VER: VER(id, operation); break;
                case Operations.USR: USR(id, operation); break;
                case Operations.BYE: BYE(id, operation); break;
                case Operations.FIL: FIL(id, operation); break;
                case Operations.MSG: distribute(id, operation); break;
                case Operations.SST: distribute(id, operation); break;
                case Operations.SET: distribute(id, operation); break;
                case Operations.REQ: REQ(id, operation); break;
                case Operations.DEL: distribute(id, operation); break;
                case Operations.SHA: distribute(id, operation); break;
                case Operations.POS: distribute(id, operation); break;
                case Operations.PLA: distribute(id, operation); break;
                case Operations.TAK: TAK(id, operation); break;
            }
        }

        private void TAK(int id, NetworkOperation op)
        {
            int receptorid = Convert.ToInt32(op.GetParameter(3));
            _myServer.Send(receptorid, op, false);
        }

        private void REQ(int id, NetworkOperation op)
        {
            if ((_receptor.Subtitle.Paragraphs.Count > 0))
            {
                foreach (Paragraph seq in _receptor.Subtitle.Paragraphs)
                {
                    StringCollection sc = new StringCollection();

                    foreach (string line in seq.Text.Split(Environment.NewLine.ToCharArray(), StringSplitOptions.RemoveEmptyEntries))
                        sc.Add(line);
                    _myServer.Send(id, SwgClient.createSEQMsg(0, _receptor.Subtitle.GetIndex(seq), (int)seq.StartTime.TotalMilliseconds, (int)seq.EndTime.TotalMilliseconds, sc, true), false);
                }

                if (_receptor.TranslatedSubtitle.Paragraphs.Count > 0)
                {
                    foreach (Paragraph seq in _receptor.TranslatedSubtitle.Paragraphs)
                    {
                        StringCollection sc = new StringCollection();
                        foreach (string line in seq.Text.Split(Environment.NewLine.ToCharArray(), StringSplitOptions.RemoveEmptyEntries))
                            sc.Add(line);
                        _myServer.Send(id, SwgClient.CreateTEXMessage(0, _receptor.TranslatedSubtitle.GetIndex(seq), sc, true), false);
                    }
                }
            }
        }

        private void VER(int id, NetworkOperation op)
        {
            if (op.ParametersLine().IndexOf("SWP1") > -1)
            {
                op.ClearParameters();
                op.AddParameter("SWP1", true);
                _myServer.Send(id, op, false);
            }
            else
                _myServer.DisconnectClient(id);
        }

        private void USR(int id, NetworkOperation op)
        {
            //-->USR 0 [nick]
            string nick = op.GetDecodedParameter(2);
            _users.Add(nick, id);
            op.ClearParameters();
            if (_fileName == "")
            {
                op.AddParameter("OK", true);
                op.AddParameter(id.ToString(), false);
            }
            else
            {
                op.AddParameter("OK", true);
                op.AddParameter(id.ToString(), false);
                op.AddParameter(_fileName, true);
                op.AddParameter(_fileSize.ToString(), false);
            }
            //<--USR 0 [filename]|OK ([filesize])
            _myServer.Send(id, op, false);

            NetworkCommand JOI = new NetworkCommand("JOI", 0);
            JOI.AddParameter(id.ToString(), true);
            JOI.AddParameter(nick, true);
            //<--JOI 0 [id] [nick]
            _myServer.sendToAllExceptOne(id, JOI);

            foreach (User u in _users)
            {
                if (u.ConnectionId != id)
                {
                    NetworkCommand LST = new NetworkCommand("LST", 0);
                    LST.AddParameter(u.ConnectionId.ToString(), false);
                    LST.AddParameter(u.Nick, true);
                    _myServer.Send(id, LST, false);
                    //<-- LST 0 [conn] [nick]
                }
            }

            _myServer.Send(id, GetINI(_receptor.Subtitle.Paragraphs.Count), false);
//            _myServer.Send(id, GetINI(1), false);
        }

        private NetworkCommand GetINI(int count)
        {
            NetworkCommand INI = new NetworkCommand("INI", 0);
            INI.AddParameter(count.ToString(), false);
            return INI;
        }

        private void ClientDisconnects(int id)
        {
            _users.delUser(id);
            NetworkCommand BYE = new NetworkCommand("BYE", 0, id.ToString());
            if (_myServer != null)
                _myServer.sendToAll(BYE);
        }

        private void BYE(int id, NetworkOperation op)
        {
            _myServer.DisconnectClient(id);
        }

        private void FIL(int id, NetworkOperation op)
        {
            if (op.CountParameters() > 3)
            {
                _fileName = op.GetDecodedParameter(2);
                _fileSize = Convert.ToInt64(op.GetParameter(3));
                _myServer.sendToAllExceptOne(id, op);
            }
        }

        private void distribute(int id, NetworkOperation op)
        {
            _myServer.sendToAllExceptOne(id, op);
        }

        public void Close()
        {
            if (_myServer != null)
            {
                _myServer.Stop();
                _myServer = null;
            }
        }

        public void Initialize(int count)
        {
            _myServer.sendToAll(GetINI(count));
        }

    }
}
