﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace Nikse.SubtitleEdit.Logic.SubtitleFormats
{
    public class RadiantGridCaptionXml : SubtitleFormat
    {

        public override string Extension
        {
            get { return ".cxml"; }
        }

        public override string Name
        {
            get { return "RadiantGrid Caption Xml"; }
        }

        public override bool IsTimeBased
        {
            get { return true; }
        }

        public override bool IsMine(List<string> lines, string fileName)
        {
            var sb = new StringBuilder();
            lines.ForEach(line => sb.AppendLine(line));
            string xmlAsString = sb.ToString().Trim();
            if ((xmlAsString.Contains("<captions") || xmlAsString.Contains("</caption>")) && (xmlAsString.Contains("</captions>")))
            {

                var subtitle = new Subtitle();
                LoadSubtitle(subtitle, lines, fileName);
                return subtitle.Paragraphs.Count > _errorCount;
            }
            return false;
        }


        public override string ToText(Subtitle subtitle, string title)
        {
            const string Layout = "<captions xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xmlns:xsd='http://www.w3.org/2001/XMLSchema' frame-rate='29.97003' duration='00:00:37.6709668' is-padding='false' caption-count='1129' frame-count='1129' starting-timecode='00:59:56;12'></captions>";
            string xmlStructure = Layout.Replace("'", "\"");

            var xml = new XmlDocument();
            xml.LoadXml(xmlStructure);
            XmlNode captionsRoot = xml.DocumentElement;
            captionsRoot.InnerXml = string.Empty;
            int no = 0;
            foreach (Paragraph p in subtitle.Paragraphs)
            {
                XmlNode caption = xml.CreateElement("caption");

                XmlAttribute start = xml.CreateAttribute("time");
                start.InnerText = p.StartTime.ToString().Replace(",", ".");
                caption.Attributes.Append(start);

                p.CalculateFrameNumbersFromTimeCodes(Configuration.Settings.General.CurrentFrameRate);
                XmlAttribute end = xml.CreateAttribute("frame");
                end.InnerText = p.StartFrame.ToString();
                caption.Attributes.Append(end);


                XmlNode eia608List = xml.CreateElement("caption-eia608-list");
                caption.AppendChild(eia608List);


                XmlNode atsc708List = xml.CreateElement("caption-atsc708-list");
                caption.AppendChild(atsc708List);


                XmlNode smpte334mList = xml.CreateElement("caption-smpte334m-list");
                caption.AppendChild(smpte334mList);


                XmlNode timecodeList = xml.CreateElement("caption-timecode-list");
                caption.AppendChild(timecodeList);

                
                captionsRoot.AppendChild(caption);
                no++;
            }

            string s = ToUtf8XmlString(xml, true);
            return s.Trim();
        }

        public override void LoadSubtitle(Subtitle subtitle, List<string> lines, string fileName)
        {
            _errorCount = 0;

            var sb = new StringBuilder();
            lines.ForEach(line => sb.AppendLine(line));
            var xml = new XmlDocument();
            try
            {
                xml.LoadXml(sb.ToString().Trim());
            }
            catch
            {
                _errorCount++;
                return;
            }

            foreach (XmlNode node in xml.DocumentElement.SelectNodes("caption"))
            {
                var p = new Paragraph();
                XmlAttribute frame = node.Attributes["frame"];
                if (frame != null)
                {
                    p.StartFrame = Convert.ToInt32(frame.InnerText);
                    p.CalculateTimeCodesFromFrameNumbers(Configuration.Settings.General.CurrentFrameRate);
                    foreach (XmlNode smpte334m in node.SelectNodes("caption-smpte334m-list/caption-smpte334m"))
                    {
                        XmlNode bytes = smpte334m.SelectSingleNode("bytes");
                        if (bytes != null)
                        {
                            int errors = 0;
                            
                            p.Text = ScenaristClosedCaptions.GetSccText(bytes.InnerText, ref errors);
                        }
                    }
                    subtitle.Paragraphs.Add(p);
                }
            }
            subtitle.Renumber(1);
        }
        
        private static TimeCode GetTimeCode(string s)
        {
            if (s.EndsWith("s"))
            {
                s = s.TrimEnd('s');
                var ts = TimeSpan.FromSeconds(double.Parse(s));
                return new TimeCode(ts);
            }
            else
            {
                string[] parts = s.Split(new char[] { ':', '.', ',' });
                var ts = new TimeSpan(0, int.Parse(parts[0]), int.Parse(parts[1]), int.Parse(parts[2]), int.Parse(parts[3]));
                return new TimeCode(ts);
            }
        }
    }
}


